@@ -9,6 +9,7 @@ use arti_client::{
 };
 use derive_deftly::Deftly;
 use std::{net::IpAddr, sync::Arc};
+use tor_error::into_internal;
 use tor_rtcompat::Runtime;
 
 use tor_rpcbase::{self as rpc, static_rpc_invoke_fn, templates::*};
@@ -24,6 +25,17 @@ use tor_rpcbase::{self as rpc, static_rpc_invoke_fn, templates::*};
 ///
 ///  * Typically, after authentication, there is one `RpcSession` for the `Connection`.
 ///    But a client may authenticate more than once; each time produces a new `RpcSession`.
+///
+/// ## In the arti rpc system
+///
+/// Base type for an authenticated RPC session.
+///
+/// Upon successful authentication via `auth:authenticate`,
+/// a connection will return either a Session object of this type,
+/// or a Session object that wraps this type.
+/// All other useful objects are available via an RPC session.
+///
+/// This ObjectID for this object can be used as the target of a SOCKS stream.
 #[derive(Deftly)]
 #[derive_deftly(Object)]
 #[deftly(rpc(expose_outside_of_session))]
@@ -66,71 +78,12 @@ impl RpcSession {
     }
 }
 
-/// RPC method to release a single strong reference.
-#[derive(Debug, serde::Deserialize, Deftly)]
-#[derive_deftly(DynMethod)]
-#[deftly(rpc(method_name = "rpc:release"))]
-struct RpcRelease {
-    /// The object to release. Must be a strong reference.
-    ///
-    /// TODO RPC: Releasing a weak reference is perilous and hard-to-define
-    /// based on how we have implemented our object ids.  If you tell the objmap
-    /// to "release" a single name for a weak reference, you are releasing every
-    /// name for that weak reference, which may have surprising results.
-    ///
-    /// This might be a sign of a design problem.
-    obj: rpc::ObjectId,
-}
-/// RPC method to release a single strong reference, creating a weak reference
-/// in its place.
-#[derive(Debug, serde::Serialize, serde::Deserialize)]
-#[allow(unused)] //TODO RPC
-struct RpcDowngrade {
-    /// The object to downgrade
-    obj: rpc::ObjectId,
-}
-
-impl rpc::RpcMethod for RpcRelease {
-    type Output = rpc::Nil;
-    type Update = rpc::NoUpdates;
-}
-
-/// Implementation for calling "release" on a Session.
-async fn rpc_release(
-    _obj: Arc<RpcSession>,
-    method: Box<RpcRelease>,
-    ctx: Arc<dyn rpc::Context>,
-) -> Result<rpc::Nil, rpc::RpcError> {
-    ctx.release_owned(&method.obj)?;
-    Ok(rpc::Nil::default())
-}
-
-/// A simple temporary method to echo a reply.
-#[derive(Debug, serde::Deserialize, serde::Serialize, Deftly)]
-#[derive_deftly(DynMethod)]
-#[deftly(rpc(method_name = "arti:x_echo"))]
-struct Echo {
-    /// A message to echo.
-    msg: String,
-}
-
-impl rpc::RpcMethod for Echo {
-    type Output = Echo;
-    type Update = rpc::NoUpdates;
-}
-
-/// Implementation for calling "echo" on a Session.
+/// Return the default client for a session.
 ///
-/// TODO RPC: Remove this. It shouldn't exist.
-async fn echo_on_session(
-    _obj: Arc<RpcSession>,
-    method: Box<Echo>,
-    _ctx: Arc<dyn rpc::Context>,
-) -> Result<Echo, rpc::RpcError> {
-    Ok(*method)
-}
-
-/// An RPC method to return the default client for a session.
+/// Allocates a new ObjectID,
+/// but does not create a new underlying client object.
+///
+/// The returned ObjectID is a handle to a `TorClient`.
 #[derive(Debug, serde::Deserialize, serde::Serialize, Deftly)]
 #[derive_deftly(DynMethod)]
 #[deftly(rpc(method_name = "arti:get_client"))]
@@ -173,7 +126,7 @@ async fn session_connect_with_prefs(
 ) -> ClientConnectionResult<arti_client::DataStream> {
     *rpc::invoke_special_method(ctx, session.client_as_object(), method)
         .await
-        .map_err(|e| Box::new(e) as _)?
+        .map_err(|e| Box::new(into_internal!("unable to delegate to TorClient")(e)) as _)?
 }
 
 /// Implement ResolveWithPrefs on an RpcSession
@@ -186,7 +139,7 @@ async fn session_resolve_with_prefs(
 ) -> ClientConnectionResult<Vec<IpAddr>> {
     *rpc::invoke_special_method(ctx, session.client_as_object(), method)
         .await
-        .map_err(|e| Box::new(e) as _)?
+        .map_err(|e| Box::new(into_internal!("unable to delegate to TorClient")(e)) as _)?
 }
 
 /// Implement ResolvePtrWithPrefs on an RpcSession
@@ -199,11 +152,9 @@ async fn session_resolve_ptr_with_prefs(
 ) -> ClientConnectionResult<Vec<String>> {
     *rpc::invoke_special_method(ctx, session.client_as_object(), method)
         .await
-        .map_err(|e| Box::new(e) as _)?
+        .map_err(|e| Box::new(into_internal!("unable to delegate to TorClient")(e)) as _)?
 }
 static_rpc_invoke_fn! {
-    rpc_release;
-    echo_on_session;
     get_client_on_session;
     isolated_client_on_session;
     @special session_connect_with_prefs;
@@ -218,12 +169,13 @@ mod list_all_methods {
     use derive_deftly::Deftly;
     use tor_rpcbase::{self as rpc, static_rpc_invoke_fn, templates::*, RpcDispatchInformation};
 
-    /// An RPC method to return a description of methods recognized on a session.
+    /// Return a description of all recognized RPC methods.
     ///
-    /// Note that not every recognized method is necessarily invocable in practice
-    /// depending on the session's access level, you might not be able to
+    /// Note that not every recognized method is necessarily invocable in practice.
+    /// Depending on the session's access level, you might not be able to
     /// access any objects that the method might be invocable upon.
     ///
+    /// **This is an experimental method.**
     /// Methods starting with "x_" are extra-unstable.
     /// See [`RpcDispatchInformation`] for caveats about type names.
     #[derive(Debug, serde::Deserialize, Deftly)]
