@@ -17,9 +17,9 @@ use arti_client::TorClientConfig;
 #[cfg(feature = "onion-service-service")]
 use tor_config::define_list_builder_accessors;
 use tor_config::resolve_alternative_specs;
-#[cfg(feature = "rpc")]
-use tor_config::CfgPath;
 pub(crate) use tor_config::{impl_standard_builder, ConfigBuildError, Listen};
+#[cfg(feature = "rpc")]
+use tor_config_path::CfgPath;
 
 use crate::{LoggingConfig, LoggingConfigBuilder};
 
@@ -92,6 +92,11 @@ impl_standard_builder! { ApplicationConfig }
 ///
 /// Broken out into a macro so as to avoid having to state the field name four times,
 /// which is a recipe for programming slips.
+///
+/// NOTE: Don't use this for new ports options!
+/// We only have to use it where we do because of the legacy `port` options.
+/// For new ports, provide a listener only.
+#[deprecated = "This macro is only for supporting old _port options! Don't use it for new options."]
 macro_rules! resolve_listen_port {
     { $self:expr, $field:ident, $def_port:expr } => { paste!{
         resolve_alternative_specs(
@@ -117,7 +122,10 @@ macro_rules! resolve_listen_port {
 #[allow(clippy::option_option)] // Builder port fields: Some(None) = specified to disable
 pub struct ProxyConfig {
     /// Addresses to listen on for incoming SOCKS connections.
-    #[builder(field(build = r#"resolve_listen_port!(self, socks, 9150)"#))]
+    #[builder(field(build = r#"#[allow(deprecated)]
+                   // We use this deprecated macro to instantiate the legacy socks_port option.
+                   { resolve_listen_port!(self, socks, 9150) }
+                 "#))]
     pub(crate) socks_listen: Listen,
 
     /// Port to listen on (at localhost) for incoming SOCKS connections.
@@ -133,7 +141,10 @@ pub struct ProxyConfig {
     pub(crate) socks_port: (),
 
     /// Addresses to listen on for incoming DNS connections.
-    #[builder(field(build = r#"resolve_listen_port!(self, dns, 0)"#))]
+    #[builder(field(build = r#"#[allow(deprecated)]
+                   // We use this deprecated macro to instantiate the legacy dns_port option.
+                   { resolve_listen_port!(self, dns, 0) }
+                 "#))]
     pub(crate) dns_listen: Listen,
 
     /// Port to listen on (at localhost) for incoming DNS connections.
@@ -272,6 +283,7 @@ impl_standard_builder! { ArtiConfig }
 impl ArtiConfigBuilder {
     /// Build the [`ArtiConfig`].
     pub fn build(&self) -> Result<ArtiConfig, ConfigBuildError> {
+        #[cfg_attr(not(feature = "onion-service-service"), allow(unused_mut))]
         let mut config = self.build_unvalidated()?;
         #[cfg(feature = "onion-service-service")]
         for svc in config.onion_services.values_mut() {
@@ -636,6 +648,18 @@ mod test {
             ],
         );
 
+        // These are commented-out by default
+        declare_exceptions(
+            None,
+            None,
+            FeatureDependent,
+            &[
+                "storage.keystore.ctor",
+                "storage.keystore.ctor.services",
+                "storage.keystore.ctor.clients",
+            ],
+        );
+
         out.sort();
 
         let dupes = out.iter().map(|exc| &exc.key).duplicates().collect_vec();
@@ -1149,7 +1173,7 @@ example config file {which:?}, uncommented={uncommented:?}
         #[cfg(feature = "pt-client")]
         {
             use arti_client::config::{pt::TransportConfig, BridgesConfig};
-            use tor_config::CfgPath;
+            use tor_config_path::CfgPath;
 
             let bridges_got: &BridgesConfig = cfg_got.0.as_ref();
 
@@ -1215,11 +1239,16 @@ example config file {which:?}, uncommented={uncommented:?}
                     .unwrap();
                 let inner_defaulted_low = defaulted_low.inner().unwrap();
                 assert_eq!(inner, inner_defaulted_low);
-            } else {
+            } else if #[cfg(arti_features_precise)] {
                 // Test that requesting memory quota tracking generates a config error
                 // if support is compiled out.
                 let m = result.unwrap_err().report().to_string();
                 assert!(m.contains("cargo feature `memquota` disabled"), "{m:?}");
+            } else {
+                // The `tor-memquota/memquota` feature is enabled by default in tor-memquota,
+                // but the corresponding `memquota` feature is but not enabled here in `arti`.
+                // so cargo --workspace enables it in a way we can't tell.  See arti/build.rs.
+                println!("not testing memquota config, cannot figure out if it's enabled");
             }
         }
     }
@@ -1475,7 +1504,7 @@ example config file {which:?}, uncommented={uncommented:?}
 
     #[test]
     fn builder() {
-        use tor_config::CfgPath;
+        use tor_config_path::CfgPath;
         let sec = std::time::Duration::from_secs(1);
 
         let auth = dir::Authority::builder()
