@@ -205,7 +205,7 @@ This could be more efficient in space and time.
         ///
         /// Return None if there is no such element.
         ///
-        /// # Safety
+        /// # Correctness hazard!
         ///
         /// This function can put this set into an inconsistent state if the
         /// mutable reference is used to change any of the keys. Doing this does
@@ -215,7 +215,7 @@ This could be more efficient in space and time.
         ///
         /// If you cannot prove to yourself that this won't happen, then you
         /// should use `modify_by_*` instead.
-        $vis unsafe fn [<by_ $key _mut>] <BorrowAsKey_>(
+        $vis fn [<by_ $key _mut_hazardous>] <BorrowAsKey_>(
             &mut self,
             key: &BorrowAsKey_
         ) -> Option<&mut $V>
@@ -238,8 +238,7 @@ This could be more efficient in space and time.
 
         #[doc = concat!("Remove the element whose `", stringify!($key), "` is `key`")]
         ///
-        /// Return that element on success, and None if there is no such element.")]
-        #[doc=stringify!($key)]
+        /// Return that element on success, and None if there is no such element.
         $vis fn [<remove_by_ $key>] <BorrowAsKey_>(&mut self, $key: &BorrowAsKey_) -> Option<$V>
             where $KEY : std::borrow::Borrow<BorrowAsKey_>,
                   BorrowAsKey_: std::hash::Hash + Eq + ?Sized
@@ -286,7 +285,7 @@ This could be more efficient in space and time.
             self.values.iter().map(|(_, v)| v)
         }
 
-        /// Consumer this container and return an iterator of its values.
+        /// Consume this container and return an iterator of its values.
         $vis fn into_values(self) -> impl Iterator<Item=$V> {
             self.values.into_iter().map(|(_, v)| v)
         }
