@@ -50,6 +50,7 @@ mod mgr;
 #[cfg(test)]
 mod testing;
 pub mod transport;
+pub(crate) mod util;
 
 use futures::select_biased;
 use futures::task::SpawnExt;
@@ -62,6 +63,9 @@ use tor_error::error_report;
 use tor_linkspec::{ChanTarget, OwnedChanTarget};
 use tor_netdir::{params::NetParameters, NetDirProvider};
 use tor_proto::channel::Channel;
+#[cfg(feature = "experimental-api")]
+use tor_proto::memquota::ChannelAccount;
+use tor_proto::memquota::ToplevelAccount;
 use tracing::debug;
 use void::{ResultVoidErrExt, Void};
 
@@ -188,6 +192,20 @@ pub enum ChannelUsage {
 impl<R: Runtime> ChanMgr<R> {
     /// Construct a new channel manager.
     ///
+    /// A new `ChannelAccount` will be made from `memquota`, for each Channel.
+    ///
+    /// The `ChannelAccount` is used for data associated with this channel.
+    ///
+    /// This does *not* (currently) include downstream outbound data
+    /// (ie, data processed by the channel implementation here,
+    /// awaiting TLS processing and actual transmission).
+    /// In any case we try to keep those buffers small.
+    ///
+    /// The ChannelAccount *does* track upstream outbound data
+    /// (ie, data processed by a circuit, but not yet by the channel),
+    /// even though that data relates to a specific circuit.
+    /// TODO #1652 use `CircuitAccount` for circuit->channel queue.
+    ///
     /// # Usage note
     ///
     /// For the manager to work properly, you will need to call `ChanMgr::launch_background_tasks`.
@@ -196,6 +214,7 @@ impl<R: Runtime> ChanMgr<R> {
         config: &ChannelConfig,
         dormancy: Dormancy,
         netparams: &NetParameters,
+        memquota: ToplevelAccount,
     ) -> Self
     where
         R: 'static,
@@ -210,7 +229,8 @@ impl<R: Runtime> ChanMgr<R> {
             #[cfg(feature = "pt-client")]
             None,
         );
-        let mgr = mgr::AbstractChanMgr::new(factory, config, dormancy, netparams, reporter);
+        let mgr =
+            mgr::AbstractChanMgr::new(factory, config, dormancy, netparams, reporter, memquota);
         ChanMgr {
             mgr,
             bootstrap_status: receiver,
@@ -244,9 +264,12 @@ impl<R: Runtime> ChanMgr<R> {
         Ok(vec![handle])
     }
 
-    /// Build a channel for an incoming stream. The channel may or may not be authenticated. This
-    /// method will wait until the channel is usable, and may return an error if we already have an
-    /// existing channel to this peer, or if there are already too many open connections with this
+    /// Build a channel for an incoming stream.
+    ///
+    /// The channel may or may not be authenticated.
+    /// This method will wait until the channel is usable,
+    /// and may return an error if we already have an existing channel to this peer,
+    /// or if there are already too many open connections with this
     /// peer or subnet (as a dos defence).
     #[cfg(feature = "relay")]
     pub async fn handle_incoming(
@@ -310,10 +333,17 @@ impl<R: Runtime> ChanMgr<R> {
         how: tor_config::Reconfigure,
         netparams: Arc<dyn AsRef<NetParameters>>,
     ) -> StdResult<(), ReconfigureError> {
+        if how == tor_config::Reconfigure::CheckAllOrNothing {
+            // Since `self.mgr.reconfigure` returns an error type of `Bug` and not
+            // `ReconfigureError` (see check below), the reconfigure should only fail due to bugs.
+            // This means we can return `Ok` here since there should never be an error with the
+            // provided `config` values.
+            return Ok(());
+        }
+
         let r = self.mgr.reconfigure(config, netparams);
 
-        // We don't care about how, because reconfiguration can only fail due to bugs
-        let _ = how;
+        // Check that `self.mgr.reconfigure` returns an error type of `Bug` (see comment above).
         let _: Option<&tor_error::Bug> = r.as_ref().err();
 
         Ok(r?)
@@ -346,6 +376,7 @@ impl<R: Runtime> ChanMgr<R> {
     pub async fn build_unmanaged_channel(
         &self,
         target: impl tor_linkspec::IntoOwnedChanTarget,
+        memquota: ChannelAccount,
     ) -> Result<Arc<Channel>> {
         use factory::ChannelFactory as _;
         let target = target.to_owned();
@@ -353,7 +384,7 @@ impl<R: Runtime> ChanMgr<R> {
         self.mgr
             .channels
             .builder()
-            .connect_via_transport(&target, self.mgr.reporter.clone())
+            .connect_via_transport(&target, self.mgr.reporter.clone(), memquota)
             .await
     }
 
