@@ -46,12 +46,14 @@ mod err;
 pub mod file_watcher;
 mod flatten;
 pub mod list_builder;
+mod listen;
 pub mod load;
 mod misc;
 pub mod mistrust;
 mod mut_cfg;
-mod path;
 pub mod sources;
+#[cfg(feature = "testing")]
+pub mod testing;
 
 #[doc(hidden)]
 pub mod deps {
@@ -68,10 +70,10 @@ pub use cmdline::CmdLine;
 pub use err::{ConfigBuildError, ConfigError, ReconfigureError};
 pub use flatten::{Flatten, Flattenable};
 pub use list_builder::{MultilineListBuilder, MultilineListBuilderError};
+pub use listen::*;
 pub use load::{resolve, resolve_ignore_warnings, resolve_return_results};
 pub use misc::*;
 pub use mut_cfg::MutCfg;
-pub use path::{CfgPath, CfgPathError};
 pub use sources::{ConfigurationSource, ConfigurationSources};
 
 use itertools::Itertools;
