@@ -61,7 +61,6 @@ mod ipt_lid;
 mod ipt_mgr;
 mod ipt_set;
 mod keys;
-mod netdir;
 mod publish;
 mod rend_handshake;
 mod replay;
@@ -91,7 +90,6 @@ use internal_prelude::*;
 
 // ---------- public exports ----------
 
-pub use crate::netdir::NetdirProviderShutdown;
 pub use anon_level::Anonymity;
 pub use config::OnionServiceConfig;
 pub use err::{ClientError, EstablishSessionError, FatalError, IntroRequestError, StartupError};
@@ -248,6 +246,7 @@ impl OnionService {
         runtime: R,
         netdir_provider: Arc<dyn NetDirProvider>,
         circ_pool: Arc<HsCircPool<R>>,
+        path_resolver: Arc<tor_config_path::CfgPathResolver>,
     ) -> Result<(Arc<RunningOnionService>, impl Stream<Item = RendRequest>), StartupError>
     where
         R: Runtime,
@@ -327,6 +326,7 @@ impl OnionService {
             config_rx,
             status_tx.clone().into(),
             Arc::clone(&keymgr),
+            path_resolver,
         );
 
         let svc = Arc::new(RunningOnionService {
