@@ -76,6 +76,7 @@
 use tor_config::file_watcher::{
     self, Event as FileEvent, FileEventReceiver, FileEventSender, FileWatcher, FileWatcherBuilder,
 };
+use tor_config_path::{CfgPath, CfgPathResolver};
 use tor_netdir::{DirEvent, NetDir};
 
 use crate::config::restricted_discovery::{
@@ -179,6 +180,8 @@ pub(super) struct Reactor<R: Runtime, M: Mockable> {
     ///
     /// Closing this channel will cause any pending upload tasks to be dropped.
     shutdown_tx: broadcast::Sender<Void>,
+    /// Path resolver for configuration files.
+    path_resolver: Arc<CfgPathResolver>,
 }
 
 /// The immutable, shared state of the descriptor publisher reactor.
@@ -547,6 +550,7 @@ impl<R: Runtime, M: Mockable> Reactor<R, M> {
         config_rx: watch::Receiver<Arc<OnionServiceConfig>>,
         status_tx: PublisherStatusSender,
         keymgr: Arc<KeyMgr>,
+        path_resolver: Arc<CfgPathResolver>,
     ) -> Self {
         /// The maximum size of the upload completion notifier channel.
         ///
@@ -555,15 +559,17 @@ impl<R: Runtime, M: Mockable> Reactor<R, M> {
         /// each sender will send exactly one message.
         const UPLOAD_CHAN_BUF_SIZE: usize = 0;
 
+        // Internally-generated instructions, no need for mq.
         let (upload_task_complete_tx, upload_task_complete_rx) =
-            mpsc::channel(UPLOAD_CHAN_BUF_SIZE);
+            mpsc_channel_no_memquota(UPLOAD_CHAN_BUF_SIZE);
 
         let (publish_status_tx, publish_status_rx) = watch::channel();
         // Setting the buffer size to zero here is OK,
         // since we never actually send anything on this channel.
         let (shutdown_tx, _shutdown_rx) = broadcast::channel(0);
 
-        let authorized_clients = Self::read_authorized_clients(&config.restricted_discovery);
+        let authorized_clients =
+            Self::read_authorized_clients(&config.restricted_discovery, &path_resolver);
 
         // Create a channel for watching for changes in the configured
         // restricted_discovery.key_dirs.
@@ -600,6 +606,7 @@ impl<R: Runtime, M: Mockable> Reactor<R, M> {
             upload_task_complete_rx,
             upload_task_complete_tx,
             shutdown_tx,
+            path_resolver,
         }
     }
 
@@ -613,7 +620,10 @@ impl<R: Runtime, M: Mockable> Reactor<R, M> {
         debug!(nickname=%self.imm.nickname, "starting descriptor publisher reactor");
 
         {
-            let netdir = wait_for_netdir(self.dir_provider.as_ref(), Timeliness::Timely).await?;
+            let netdir = self
+                .dir_provider
+                .wait_for_netdir(Timeliness::Timely)
+                .await?;
             let time_periods = self.compute_time_periods(&netdir, &[])?;
 
             let mut inner = self.inner.lock().expect("poisoned lock");
@@ -740,7 +750,7 @@ impl<R: Runtime, M: Mockable> Reactor<R, M> {
                         //
                         // Probably this should be fixed by moving the logging
                         // out of the reactor, where it won't be blocked.
-                        wait_for_netdir(self.dir_provider.as_ref(), Timeliness::Timely)
+                        self.dir_provider.wait_for_netdir(Timeliness::Timely)
                             .await?
                     }
                 };
@@ -1024,7 +1034,7 @@ impl<R: Runtime, M: Mockable> Reactor<R, M> {
 
             let dirs = inner.config.restricted_discovery.key_dirs().clone();
 
-            watch_dirs(&mut watcher, &dirs);
+            watch_dirs(&mut watcher, &dirs, &self.path_resolver);
 
             let watcher = watcher
                 .start_watching(self.key_dirs_tx.clone())
@@ -1199,7 +1209,8 @@ impl<R: Runtime, M: Mockable> Reactor<R, M> {
     /// Returns `true` if the authorized clients have changed.
     async fn update_authorized_clients_if_changed(&mut self) -> Result<bool, FatalError> {
         let mut inner = self.inner.lock().expect("poisoned lock");
-        let authorized_clients = Self::read_authorized_clients(&inner.config.restricted_discovery);
+        let authorized_clients =
+            Self::read_authorized_clients(&inner.config.restricted_discovery, &self.path_resolver);
 
         let clients = &mut inner.authorized_clients;
         let changed = clients.as_ref() != authorized_clients.as_ref();
@@ -1215,8 +1226,9 @@ impl<R: Runtime, M: Mockable> Reactor<R, M> {
     /// Read the authorized `RestrictedDiscoveryKeys` from `config`.
     fn read_authorized_clients(
         config: &RestrictedDiscoveryConfig,
+        path_resolver: &CfgPathResolver,
     ) -> Option<Arc<RestrictedDiscoveryKeys>> {
-        let authorized_clients = config.read_keys();
+        let authorized_clients = config.read_keys(path_resolver);
 
         if matches!(authorized_clients.as_ref(), Some(c) if c.is_empty()) {
             warn!(
@@ -1785,6 +1797,10 @@ impl<R: Runtime, M: Mockable> Reactor<R, M> {
     /// Returns `Ok(None)` if restricted discovery mode is disabled.
     ///
     /// Returns an error if restricted discovery mode is enabled, but the client list is empty.
+    #[cfg_attr(
+        not(feature = "restricted-discovery"),
+        allow(clippy::unnecessary_wraps)
+    )]
     fn authorized_clients(&self) -> Result<Option<Arc<RestrictedDiscoveryKeys>>, FatalError> {
         cfg_if::cfg_if! {
             if #[cfg(feature = "restricted-discovery")] {
@@ -1808,10 +1824,10 @@ impl<R: Runtime, M: Mockable> Reactor<R, M> {
 }
 
 /// Try to expand a path, logging a warning on failure.
-fn maybe_expand_path(p: &tor_config::CfgPath) -> Option<PathBuf> {
+fn maybe_expand_path(p: &CfgPath, r: &CfgPathResolver) -> Option<PathBuf> {
     // map_err returns unit for clarity
     #[allow(clippy::unused_unit, clippy::semicolon_if_nothing_returned)]
-    p.path()
+    p.path(r)
         .map_err(|e| {
             tor_error::warn_report!(e, "invalid path");
             ()
@@ -1832,10 +1848,14 @@ macro_rules! watch_path {
 
 /// Add the specified directories to the watcher.
 #[allow(clippy::cognitive_complexity)]
-fn watch_dirs<R: Runtime>(watcher: &mut FileWatcherBuilder<R>, dirs: &DirectoryKeyProviderList) {
+fn watch_dirs<R: Runtime>(
+    watcher: &mut FileWatcherBuilder<R>,
+    dirs: &DirectoryKeyProviderList,
+    path_resolver: &CfgPathResolver,
+) {
     for path in dirs {
         let path = path.path();
-        let Some(path) = maybe_expand_path(path) else {
+        let Some(path) = maybe_expand_path(path, path_resolver) else {
             warn!("failed to expand key_dir path {:?}", path);
             continue;
         };
