@@ -64,10 +64,10 @@ pub use err::{
     ArtiPathSyntaxError, Error, KeystoreCorruptionError, KeystoreError, UnknownKeyTypeError,
 };
 pub use key_specifier::{
-    ArtiPathUnavailableError, CTorPath, InvalidKeyPathComponentValue, KeyPath, KeyPathError,
-    KeyPathInfo, KeyPathInfoBuilder, KeyPathInfoExtractor, KeyPathPattern, KeyPathRange,
-    KeySpecifier, KeySpecifierComponent, KeySpecifierComponentViaDisplayFromStr,
-    KeySpecifierPattern,
+    ArtiPathRange, ArtiPathUnavailableError, CTorPath, CTorServicePath,
+    InvalidKeyPathComponentValue, KeyPath, KeyPathError, KeyPathInfo, KeyPathInfoBuilder,
+    KeyPathInfoExtractor, KeyPathPattern, KeySpecifier, KeySpecifierComponent,
+    KeySpecifierComponentViaDisplayFromStr, KeySpecifierPattern,
 };
 
 #[cfg(feature = "keymgr")]
@@ -86,6 +86,10 @@ pub use {
 )]
 pub use keystore::ephemeral::ArtiEphemeralKeystore;
 
+#[cfg(all(feature = "keymgr", feature = "ctor-keystore"))]
+#[cfg_attr(docsrs, doc(cfg(all(feature = "keymgr", feature = "ctor-keystore"))))]
+pub use keystore::ctor::{CTorClientKeystore, CTorServiceKeystore};
+
 #[doc(hidden)]
 pub use key_specifier::derive as key_specifier_derive;
 
