@@ -28,6 +28,7 @@ use self::middle::HsDescMiddle;
 use self::outer::HsDescOuter;
 
 use super::desc_enc::{HsDescEncNonce, HsDescEncryption, HS_DESC_ENC_NONCE_LEN};
+use super::pow::PowParams;
 
 /// An intermediary type for encoding hidden service descriptors.
 ///
@@ -60,6 +61,10 @@ struct HsDesc<'a> {
     intro_auth_key_cert_expiry: SystemTime,
     /// The expiration time of an introduction point encryption key certificate.
     intro_enc_key_cert_expiry: SystemTime,
+    /// Proof-of-work parameters.
+    #[builder(default)]
+    #[cfg(feature = "hs-pow-full")]
+    pow_params: Option<&'a PowParams>,
     /// The list of clients authorized to discover the hidden service.
     ///
     /// If `None`, restricted discovery is disabled.
@@ -156,6 +161,8 @@ impl<'a> NetdocBuilder for HsDescBuilder<'a> {
             intro_points: hs_desc.intro_points,
             intro_auth_key_cert_expiry: hs_desc.intro_auth_key_cert_expiry,
             intro_enc_key_cert_expiry: hs_desc.intro_enc_key_cert_expiry,
+            #[cfg(feature = "hs-pow-full")]
+            pow_params: hs_desc.pow_params,
         }
         .build_sign(rng)?;
 
