@@ -9,10 +9,11 @@ use tor_error::internal;
 
 use crate::channel::codec::{self, ChannelCodec, CodecError};
 use crate::channel::UniqId;
+use crate::memquota::ChannelAccount;
 use crate::util::skew::ClockSkew;
 use crate::{Error, Result};
 use tor_cell::chancell::{msg, ChanCmd, ChanMsg};
-use tor_rtcompat::SleepProvider;
+use tor_rtcompat::{CoarseTimeProvider, SleepProvider};
 
 use std::net::SocketAddr;
 use std::sync::Arc;
@@ -36,11 +37,14 @@ static LINK_PROTOCOLS: &[u16] = &[4, 5];
 /// A raw client channel on which nothing has been done.
 pub struct OutboundClientHandshake<
     T: AsyncRead + AsyncWrite + Send + Unpin + 'static,
-    S: SleepProvider,
+    S: CoarseTimeProvider + SleepProvider,
 > {
     /// Runtime handle (insofar as we need it)
     sleep_prov: S,
 
+    /// Memory quota account
+    memquota: ChannelAccount,
+
     /// Underlying TLS stream.
     ///
     /// (We don't enforce that this is actually TLS, but if it isn't, the
@@ -57,9 +61,14 @@ pub struct OutboundClientHandshake<
 /// A client channel on which versions have been negotiated and the
 /// relay's handshake has been read, but where the certs have not
 /// been checked.
-pub struct UnverifiedChannel<T: AsyncRead + AsyncWrite + Send + Unpin + 'static, S: SleepProvider> {
+pub struct UnverifiedChannel<
+    T: AsyncRead + AsyncWrite + Send + Unpin + 'static,
+    S: CoarseTimeProvider + SleepProvider,
+> {
     /// Runtime handle (insofar as we need it)
     sleep_prov: S,
+    /// Memory quota account
+    memquota: ChannelAccount,
     /// The negotiated link protocol.  Must be a member of LINK_PROTOCOLS
     link_protocol: u16,
     /// The Source+Sink on which we're reading and writing cells.
@@ -87,9 +96,14 @@ pub struct UnverifiedChannel<T: AsyncRead + AsyncWrite + Send + Unpin + 'static,
 /// This type is separate from UnverifiedChannel, since finishing the
 /// handshake requires a bunch of CPU, and you might want to do it as
 /// a separate task or after a yield.
-pub struct VerifiedChannel<T: AsyncRead + AsyncWrite + Send + Unpin + 'static, S: SleepProvider> {
+pub struct VerifiedChannel<
+    T: AsyncRead + AsyncWrite + Send + Unpin + 'static,
+    S: CoarseTimeProvider + SleepProvider,
+> {
     /// Runtime handle (insofar as we need it)
     sleep_prov: S,
+    /// Memory quota account
+    memquota: ChannelAccount,
     /// The negotiated link protocol.
     link_protocol: u16,
     /// The Source+Sink on which we're reading and writing cells.
@@ -138,16 +152,22 @@ fn codec_err_to_handshake(err: CodecError) -> Error {
     }
 }
 
-impl<T: AsyncRead + AsyncWrite + Send + Unpin + 'static, S: SleepProvider>
+impl<T: AsyncRead + AsyncWrite + Send + Unpin + 'static, S: CoarseTimeProvider + SleepProvider>
     OutboundClientHandshake<T, S>
 {
     /// Construct a new OutboundClientHandshake.
-    pub(crate) fn new(tls: T, target_method: Option<ChannelMethod>, sleep_prov: S) -> Self {
+    pub(crate) fn new(
+        tls: T,
+        target_method: Option<ChannelMethod>,
+        sleep_prov: S,
+        memquota: ChannelAccount,
+    ) -> Self {
         Self {
             tls,
             target_method,
             unique_id: UniqId::new(),
             sleep_prov,
+            memquota,
         }
     }
 
@@ -307,13 +327,16 @@ impl<T: AsyncRead + AsyncWrite + Send + Unpin + 'static, S: SleepProvider>
                     target_method: self.target_method.take(),
                     unique_id: self.unique_id,
                     sleep_prov: self.sleep_prov.clone(),
+                    memquota: self.memquota.clone(),
                 })
             }
         }
     }
 }
 
-impl<T: AsyncRead + AsyncWrite + Send + Unpin + 'static, S: SleepProvider> UnverifiedChannel<T, S> {
+impl<T: AsyncRead + AsyncWrite + Send + Unpin + 'static, S: CoarseTimeProvider + SleepProvider>
+    UnverifiedChannel<T, S>
+{
     /// Return the reported clock skew from this handshake.
     ///
     /// Note that the skew reported by this function might not be "true": the
@@ -562,11 +585,14 @@ impl<T: AsyncRead + AsyncWrite + Send + Unpin + 'static, S: SleepProvider> Unver
             rsa_id,
             clock_skew: self.clock_skew,
             sleep_prov: self.sleep_prov,
+            memquota: self.memquota,
         })
     }
 }
 
-impl<T: AsyncRead + AsyncWrite + Send + Unpin + 'static, S: SleepProvider> VerifiedChannel<T, S> {
+impl<T: AsyncRead + AsyncWrite + Send + Unpin + 'static, S: CoarseTimeProvider + SleepProvider>
+    VerifiedChannel<T, S>
+{
     /// Send a 'Netinfo' message to the relay to finish the handshake,
     /// and create an open channel and reactor.
     ///
@@ -619,7 +645,7 @@ impl<T: AsyncRead + AsyncWrite + Send + Unpin + 'static, S: SleepProvider> Verif
             .build()
             .expect("OwnedChanTarget builder failed");
 
-        Ok(super::Channel::new(
+        super::Channel::new(
             self.link_protocol,
             Box::new(tls_sink),
             Box::new(tls_stream),
@@ -627,7 +653,8 @@ impl<T: AsyncRead + AsyncWrite + Send + Unpin + 'static, S: SleepProvider> Verif
             peer_id,
             self.clock_skew,
             self.sleep_prov,
-        ))
+            self.memquota,
+        )
     }
 }
 
@@ -640,6 +667,7 @@ pub(super) mod test {
 
     use super::*;
     use crate::channel::codec::test::MsgBuf;
+    use crate::util::fake_mq;
     use crate::Result;
     use tor_cell::chancell::msg;
     use tor_linkspec::OwnedChanTarget;
@@ -690,7 +718,7 @@ pub(super) mod test {
             // netinfo cell -- quite minimal.
             add_padded(&mut buf, NETINFO_PREFIX);
             let mb = MsgBuf::new(&buf[..]);
-            let handshake = OutboundClientHandshake::new(mb, None, rt.clone());
+            let handshake = OutboundClientHandshake::new(mb, None, rt.clone(), fake_mq());
             let unverified = handshake.connect(|| now).await?;
 
             assert_eq!(unverified.link_protocol, 5);
@@ -706,7 +734,7 @@ pub(super) mod test {
             buf.extend_from_slice(VPADDING);
             add_padded(&mut buf, NETINFO_PREFIX_WITH_TIME);
             let mb = MsgBuf::new(&buf[..]);
-            let handshake = OutboundClientHandshake::new(mb, None, rt.clone());
+            let handshake = OutboundClientHandshake::new(mb, None, rt.clone(), fake_mq());
             let unverified = handshake.connect(|| now).await?;
             // Correct timestamp in the NETINFO, so no skew.
             assert_eq!(unverified.clock_skew(), ClockSkew::None);
@@ -714,7 +742,7 @@ pub(super) mod test {
             // Now pretend our clock is fast.
             let now2 = now + Duration::from_secs(3600);
             let mb = MsgBuf::new(&buf[..]);
-            let handshake = OutboundClientHandshake::new(mb, None, rt.clone());
+            let handshake = OutboundClientHandshake::new(mb, None, rt.clone(), fake_mq());
             let unverified = handshake.connect(|| now2).await?;
             assert_eq!(
                 unverified.clock_skew(),
@@ -727,10 +755,10 @@ pub(super) mod test {
 
     async fn connect_err<T: Into<Vec<u8>>, S>(input: T, sleep_prov: S) -> Error
     where
-        S: SleepProvider,
+        S: CoarseTimeProvider + SleepProvider,
     {
         let mb = MsgBuf::new(input);
-        let handshake = OutboundClientHandshake::new(mb, None, sleep_prov);
+        let handshake = OutboundClientHandshake::new(mb, None, sleep_prov, fake_mq());
         handshake.connect(SystemTime::now).await.err().unwrap()
     }
 
@@ -857,6 +885,7 @@ pub(super) mod test {
             target_method: None,
             unique_id: UniqId::new(),
             sleep_prov: runtime,
+            memquota: fake_mq(),
         }
     }
 
@@ -1118,6 +1147,7 @@ pub(super) mod test {
                 rsa_id,
                 clock_skew: ClockSkew::None,
                 sleep_prov: rt,
+                memquota: fake_mq(),
             };
 
             let (_chan, _reactor) = ver.finish().await.unwrap();
