@@ -47,6 +47,8 @@
     allow(unused, unreachable_pub)
 )]
 
+#[cfg(feature = "bench")]
+pub mod bench_utils;
 pub mod channel;
 pub mod circuit;
 mod crypto;
