@@ -737,18 +737,25 @@ pub enum InvokeError {
     #[error("No implementation for provided object and method types.")]
     NoImpl,
 
+    /// Tried to call `invoke_without_dispatch` on an RPC method that _does_ support
+    /// regular RPC method dispatch.
+    #[error("Called invoke_without_dispatch on a regular RPC method")]
+    NoDispatchBypass,
+
     /// An internal problem occurred while invoking a method.
     #[error("Internal error")]
     Bug(#[from] tor_error::Bug),
 }
 
-impl tor_error::HasKind for InvokeError {
-    fn kind(&self) -> tor_error::ErrorKind {
-        use tor_error::ErrorKind as EK;
-        match self {
-            InvokeError::NoImpl => EK::RpcMethodNotFound,
-            InvokeError::Bug(e) => e.kind(),
-        }
+impl From<InvokeError> for RpcError {
+    fn from(err: InvokeError) -> Self {
+        use crate::RpcErrorKind as EK;
+        let kind = match &err {
+            InvokeError::NoImpl => EK::MethodNotImpl,
+            InvokeError::NoDispatchBypass => EK::InternalError,
+            InvokeError::Bug(_) => EK::InternalError,
+        };
+        RpcError::new(err.to_string(), kind)
     }
 }
 
@@ -768,12 +775,11 @@ pub(crate) mod test {
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
 
-    use crate::{method::RpcMethod, templates::*, DispatchTable, Method, NoUpdates};
+    use crate::{method::RpcMethod, templates::*, DispatchTable, InvokeError, Method, NoUpdates};
     use derive_deftly::Deftly;
     use futures::SinkExt;
     use futures_await_test::async_test;
     use std::sync::{Arc, RwLock};
-    use tor_error::{ErrorKind, HasKind as _};
 
     use super::UpdateSink;
 
@@ -1008,7 +1014,13 @@ pub(crate) mod test {
             let animal: Arc<dyn crate::Object> = Arc::new(obj);
             let request: Box<dyn DynMethod> = Box::new(method);
             let discard = Box::pin(futures::sink::drain().sink_err_into());
-            crate::invoke_rpc_method(Arc::clone(ctx), animal, request, discard)
+            crate::invoke_rpc_method(
+                Arc::clone(ctx),
+                &crate::ObjectId::from("AnimalIdent"),
+                animal,
+                request,
+                discard,
+            )
         }
         async fn invoke_ok<O: crate::Object, M: crate::Method>(
             table: &Arc<dyn Context>,
@@ -1160,6 +1172,10 @@ pub(crate) mod test {
 
     #[test]
     fn invoke_poorly() {
+        fn is_internal_invoke_err<T>(val: Result<T, InvokeError>) -> bool {
+            matches!(val, Err(InvokeError::Bug(_)))
+        }
+
         // Make sure that our invoker function invocations return plausible bugs warnings on
         // misuse.
         let ctx: Arc<dyn crate::Context> = Arc::new(Ctx::from(DispatchTable::from_inventory()));
@@ -1175,7 +1191,7 @@ pub(crate) mod test {
             Arc::clone(&ctx),
             discard(),
         );
-        assert!(bug.err().unwrap().kind() == ErrorKind::Internal);
+        assert!(is_internal_invoke_err(bug));
 
         // Wrong object type
         let bug = ent.invoke(
@@ -1184,14 +1200,14 @@ pub(crate) mod test {
             Arc::clone(&ctx),
             discard(),
         );
-        assert!(bug.err().unwrap().kind() == ErrorKind::Internal);
+        assert!(is_internal_invoke_err(bug));
 
         // Special: Wrong method.
         let bug = ent.invoke_special(Arc::new(Swan), Box::new(GetName), Arc::clone(&ctx));
-        assert!(bug.err().unwrap().kind() == ErrorKind::Internal);
+        assert!(is_internal_invoke_err(bug));
         // Special: Wrong object type
         let bug = ent.invoke_special(Arc::new(Wombat), Box::new(GetKids), Arc::clone(&ctx));
-        assert!(bug.err().unwrap().kind() == ErrorKind::Internal);
+        assert!(is_internal_invoke_err(bug));
     }
 
     #[test]
